
import { _decorator, Component, Node, EventTouch, Vec3, view } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('Script')
export class Script extends Component {

    @property(Node)
    sprite: Node = null;

    private touch_id: number = -1;
    private target_position: Vec3 = new Vec3();

    start() {
        this.node.on(Node.EventType.TOUCH_START, this.TouchEvent, this)
        this.node.on(Node.EventType.TOUCH_MOVE, this.TouchEvent, this)
        this.node.on(Node.EventType.TOUCH_END, this.TouchEvent, this)
        this.node.on(Node.EventType.TOUCH_CANCEL, this.TouchEvent, this)
    }

    TouchEvent(e: EventTouch) {
        if (e.type == Node.EventType.TOUCH_START) {
            if (this.touch_id == -1) {
                this.touch_id = e.getID();
                this.target_position = new Vec3(e.getUILocation().x - view.getCanvasSize().x * 0.5, e.getUILocation().y - view.getCanvasSize().y * 0.5, 0);
            }
        }

        if (e.type == Node.EventType.TOUCH_MOVE) {
            if (e.getID() == this.touch_id) {
                this.target_position = new Vec3(e.getUILocation().x - view.getCanvasSize().x * 0.5, e.getUILocation().y - view.getCanvasSize().y * 0.5, 0);
            }
        }

        if (e.type == Node.EventType.TOUCH_END || e.type == Node.EventType.TOUCH_CANCEL) {
            if (e.getID() == this.touch_id) {
                this.touch_id = -1;
                this.target_position = new Vec3();
            }
        }
    }

    update(deltaTime: number) {
        if (this.touch_id >= 0) {
            this.sprite.position = Vec3.lerp(new Vec3(), this.sprite.position, this.target_position, deltaTime);
        }
    }
}
